/* Mdulo NSL que representa la capa de remapeo dinmico del
   ambiente y del ancla dentro del mismo.
   Alejandra Barrera
   Versin: 1
   Fecha: 18 de marzo de 2005
   Versin: 3
   Fecha: 19 de febrero de 2007
*/


import nslj.src.system.*; 
import nslj.src.cmd.*; 
import nslj.src.lang.*; 
import nslj.src.math.*; 
import nslj.src.display.*; 
import java.lang.*;


public class DynamicRemappingLayer extends NslModule
{
	public NslDinInt0 currentDir;
	public NslDinInt0 nextDir;
	public NslDoutDouble1 PIps1dim;
	public NslDoutDouble2 PIps2dim;
	
	private int sizePS;
	private int dnrDim1;
	private int dnrDim2;
	private int dim1;
	private int dim2;
	private double PIps1dimAux[];
	private double PIps2dimAux[][];
	private Convolution conv;
	private int x;
	private int y;
	private boolean inicio;
	
	public DynamicRemappingLayer(String nslName, NslModule nslParent,int d1,int d2, int xInit, int yInit)
	{
		super(nslName, nslParent);
		dim1= d1;
		dim2= d2;
		dnrDim1= d2*2;
		dnrDim2= d1*2;
		sizePS= dnrDim1*dnrDim2;
		x= xInit;
		y= yInit;
		inicio= true;
		currentDir= new NslDinInt0("currentDir",this);
		nextDir= new NslDinInt0("nextDir",this);
		PIps1dim= new NslDoutDouble1("PIps1dim",this,sizePS); 
		PIps2dim= new NslDoutDouble2("PIps2dim",this,dnrDim1,dnrDim2); 
		PIps1dimAux= new double[sizePS];
		PIps2dimAux= new double[dnrDim1][dnrDim2];
		conv= new Convolution(d1,d2);
	}

        public void buildPerceptualSchema2Dim(int posX, int posY)
	{
		double d1 = 2D;
        int i1 = dnrDim1 - (posY * 2 + 2);
        int j1 = dnrDim2 - (posX * 2 + 2);                
        for(int k = 0; k < dnrDim1; k++)
        {
            for(int l = 0; l < dnrDim2; l++)
                PIps2dimAux[k][l] = Math.exp((double)(-1 * (k - i1) * (k - i1)) / (2D * (d1 * d1)) + (double)(-1 * (l - j1) * (l - j1)) / (2D * (d1 * d1)));
        }
	}
	
	public void buildPerceptualSchema1Dim()
	{
		for(int l = 0; l < dnrDim1; l++)
        {
            for(int i1 = 0; i1 < dnrDim2; i1++)
            {
                PIps1dimAux[l * dnrDim2 + i1] = PIps2dimAux[l][i1];
                if(PIps1dimAux[l * dnrDim2 + i1] < 0.01D)
                    PIps1dimAux[l * dnrDim2 + i1] = 0.0D;
                if(PIps1dimAux[l * dnrDim2 + i1] > 1.0D)
                    PIps1dimAux[l * dnrDim2 + i1] = 1.0D;
            }
        }
	}
	
	public void simRun()
	{
		if (inicio || (Rat.training && Rat.newTrial) || (Rat.training && World.trainingTrialNumber==1))
		{
                    if (Rat.training && World.trainingTrialNumber==1)
                    {
                        World.trainingTrialNumber= -1;
                    }
                    buildPerceptualSchema2Dim(x,y);
                    inicio= false;
                 }
                 else if (Rat.testing && Rat.newTrial && World.testingTrialNumber==1)
                 {
			buildPerceptualSchema2Dim(17,19);
                 }
                 else if (Rat.testing && Rat.newTrial && World.testingTrialNumber==2)
                 {
			buildPerceptualSchema2Dim(11,14);
                 }
                 else if (Rat.testing && Rat.newTrial && World.testingTrialNumber==3)
                 {
			buildPerceptualSchema2Dim(6,14);
                 }
                 else if (Rat.testing && Rat.newTrial && World.testingTrialNumber==4)
                 {
			buildPerceptualSchema2Dim(6,9);
                 }                
                 else
		 {
			conv.buildConvolution(currentDir.get(),nextDir.get(),PIps2dimAux);
		 }
		buildPerceptualSchema1Dim();
		PIps1dim.set(PIps1dimAux);
		PIps2dim.set(PIps2dimAux);
	} //simRun
}	
