/* Mdulo NSL que representa el esquema motivacional de la rata. 
   Alejandra Barrera
   Versin: 2
   Fecha: 19 de abril de 2005
*/


import nslj.src.system.*; 
import nslj.src.cmd.*; 
import nslj.src.lang.*; 
import nslj.src.math.*; 
import nslj.src.display.*; 
import java.lang.*;
import java.util.Random;

public class MotivationalSchema extends NslModule
{
	public NslDinInt0 turnToFood;
	public NslDinDouble1 affPS;
	public NslDinInt0 currentHeadAngleRat;
	public NslDinDouble1 expCycling;
	public NslDinInt1 flagCycling;
	public NslDoutInt0 AngleToGo;
        public NslDoutDouble1 wg;
        public NslDoutDouble1 curiosity;
        public NslDoutDouble1 sum;
        public NslDoutDouble1 food;
	private PerceptualSchema F1;
	private PerceptualSchema F3;
	private PerceptualSchema F4;
	private PerceptualSchema F6;
	private PerceptualSchema F7;
	private PerceptualSchema F5;
	private PerceptualSchema F14;
	private PerceptualSchema F18;
	private PerceptualSchema F19;
	private PerceptualSchema F20;
	private double u[]; 
    private double U[];  
    private double wg_mot[]; 
    private double curiosityAux[]; 
    private double foodAux[]; 
    private double tamAux[]; 
    private double wgAux[]; 
    private double sumAux[]; 
    private double affAux[]; 
    private final double WG_NOISE = 0.040000000000000001D;
    private int Angle;
    private double Gradient[] = {
        15D, 15D, 15D, 14D, 14D, 13D, 13D, 12D, 12D, 11D, 
        11D, 11D, 10D, 10D, 10D, 9D, 9D, 9D, 8D, 8D, 
        8D, 7D, 7D, 7D, 6D, 6D, 6D, 5D, 5D, 5D, 
        4D, 4D, 4D, 3D, 3D, 3D, 2D, 2D, 1.0D, 1.0D, 
        0, -1D, -1D, -2D, -2D, -3D, -3D, -3D, -4D, -4D, 
        -4D, -5D, -5D, -5D, -6D, -6D, -6D, -7D, -7D, -7D, 
        -8D, -8D, -8D, -9D, -9D, -9D, -10D, -10D, -10D, -11D, 
        -11D, -11D, -12D, -12D, -13D, -13D, -14D, -14D, -15D, -15D
    };
	private int turnToFoodAux;
	private boolean inicio;
	private final double CURIOSITY = 0.01D;
        private final double CURIOSITY_TESTING = 0.05D;
        private final double CURIOSITY_HABITUATION = 0.06D;
	private final double CORRIDOR_SLOPE = 30D;
	private int currentDir;
	private boolean flagCyclingAuxB[];
	private int flagCyclingAuxI[];
        private int antIndex= -1;
        
	public MotivationalSchema (String nslName, NslModule nslParent)
	{
		super(nslName, nslParent);
		turnToFood= new NslDinInt0("turnToFood",this);
		affPS= new NslDinDouble1("affPS",this,80);
		expCycling= new NslDinDouble1("expCycling",this,8);
		flagCycling= new NslDinInt1("flagCycling",this,8);
		currentHeadAngleRat= new NslDinInt0("currentHeadAngleRat",this);
		AngleToGo= new NslDoutInt0("AngleToGo",this);
                wg= new NslDoutDouble1("wg",this,5);
                curiosity= new NslDoutDouble1("curiosity",this,5);
                food= new NslDoutDouble1("food",this,5);
                sum= new NslDoutDouble1("sum",this,5);

		F1= new PerceptualSchema(80,1);
		F3= new PerceptualSchema(80,1);
		F4= new PerceptualSchema(80,1);
		F6= new PerceptualSchema(80,1);
		F7= new PerceptualSchema(80,1);
		F5= new PerceptualSchema(80,1);
		F14= new PerceptualSchema(80,1);
		F18= new PerceptualSchema(80,1);
		F19= new PerceptualSchema(80,1);
		F20= new PerceptualSchema(80,1);
		wg_mot= new double[80];
		u= new double[80];
		U= new double[80];
                affAux= new double[80];
                curiosityAux= new double[5];
                foodAux= new double[5];
                wgAux= new double[5];
                sumAux= new double[5];

		flagCyclingAuxB= new boolean[8];
		flagCyclingAuxI= new int[8];
		turnToFoodAux= -1;
		inicio= true;
	}

    public int angle2direction(int i)
    {
        switch(i)
        {
        case 0: 
            return 0;
        case 45:
            return 1;
        case 90:
            return 2;
        case 135: 
            return 3;
        case 180: 
            return 4;
        case 225: 
            return 5;
        case 270: 
            return 6;
        case 315: 
            return 7;
        }
        return -1;
    }
  
    public void initConditions(int turnToFoodAux, double psAff[])
    {
        double d = 4D;        
        for(int l = 0; l < 80; l++)
        {
            F6.act[l][0] = 0.0D;
        }
        switch(turnToFoodAux)
        {
            case -90:
                for(int k3 = 0; k3 < 80; k3++)
                    F6.act[k3][0] += 0.09 * Math.exp((double)(-1 * (k3 - 22) * (k3 - 22)) / (2D * (d * d)));
                break;
            case 0:
                for(int i4 = 0; i4 < 80; i4++)
                    F6.act[i4][0] += 0.09 * Math.exp((double)(-1 * (i4 - 40) * (i4 - 40)) / (2D * (d * d)));
                break;
            case 90:
                for(int k4 = 0; k4 < 80; k4++)
                    F6.act[k4][0] += 0.09 * Math.exp((double)(-1 * (k4 - 58) * (k4 - 58)) / (2D * (d * d)));
                break;
            case -1:
if (Rat.habituation)
{
    boolean aff[]= new boolean[4];
    for (int i1=0; i1<4; i1++)
        aff[i1]= false;
    for (int i1=1; i1<=4; i1++)
    switch(i1)
    {
	default:
		break;
	case 1:
		if (psAff[4]==1 && psAff[76]==1)
        		aff[0]= true;
		break;
	case 2:
		if (psAff[22]==1)
                	aff[1]= true;
		break;
	case 3:
		if (psAff[58]==1)
			aff[2]= true;
		break;					
	case 4:
		if (psAff[40]==1)
			aff[3]= true;
		break;					                                                        
    }           
    int affIndex= -1;
    Random random = new Random();
    affIndex= random.nextInt(4);
    while (aff[affIndex]==false)
	affIndex= random.nextInt(4);
    switch(affIndex)
    {
	default:
	break;
	case 0:
            for(int k4 = 0; k4 < 80; k4++)
            {
            	F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 4) * (k4 - 4)) / (2D * (d * d)));
            	F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 76) * (k4 - 76)) / (2D * (d * d)));
            }
	break;
	case 1:
	    for(int k3 = 0; k3 < 80; k3++)
        	F6.act[k3][0] += 0.04 * Math.exp((double)(-1 * (k3 - 22) * (k3 - 22)) / (2D * (d * d)));
	break;
	case 2:
	    for(int k4 = 0; k4 < 80; k4++)
	        F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 58) * (k4 - 58)) / (2D * (d * d)));
	break;
	case 3:
	    for(int k4 = 0; k4 < 80; k4++)
	         F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 40) * (k4 - 40)) / (2D * (d * d)));
	break;                                
    }		
}
               

else if (Rat.training)
{
					boolean aff[]= new boolean[4];
					for (int i1=0; i1<4; i1++)
						aff[i1]= false;
					for (int i1=1; i1<=4; i1++)
						switch(i1)
						{
							default:
							break;
							case 1:
				   				if (psAff[4]==1 && psAff[76]==1)
									aff[0]= true;
							break;
							case 2:
				   				if (psAff[22]==1)
									aff[1]= true;
							break;
							case 3: 
				   				if (psAff[58]==1)
									aff[2]= true;
							break;
							case 4:
				   				if (psAff[40]==1)
									aff[3]= true;
							break;					                                                        
						}
                                        
                                       int affIndex=-1; 
                                       if (aff[0] && !aff[1] && !aff[2] && !aff[3])
                                       {
                                           affIndex=0;
                                       }
                                       else if (aff[0] && aff[3] && !aff[1] && !aff[2])
                                       {
                                           affIndex=3;
                                       }
                                       else
                                       {
                                        affIndex= -1;
					Random random = new Random();
					affIndex= random.nextInt(4);
					while (aff[affIndex]==false)
						affIndex= random.nextInt(4);
					if (affIndex==0 && (aff[1] || aff[2] || aff[3]))
					{
						aff[0]= false;
						while (aff[affIndex]==false)
							affIndex= random.nextInt(4);
					}                                                   
                                       }

                                       switch(affIndex)
					{
						default:
						break;
						case 0:
							for(int k4 = 0; k4 < 80; k4++)
							{
                    			F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 4) * (k4 - 4)) / (2D * (d * d)));
                    			F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 76) * (k4 - 76)) / (2D * (d * d)));
                    		}
						break;
						case 1:
						    for(int k3 = 0; k3 < 80; k3++)
                    			F6.act[k3][0] += 0.04 * Math.exp((double)(-1 * (k3 - 22) * (k3 - 22)) / (2D * (d * d)));
		                break;
						case 2:
			                for(int k4 = 0; k4 < 80; k4++)
			                    F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 58) * (k4 - 58)) / (2D * (d * d)));
		                break;
                                           case 3:
                                               for(int k4 = 0; k4 < 80; k4++)
                                        F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 40) * (k4 - 40)) / (2D * (d * d)));
                                              break;
					}		

}
else if (Rat.testing)
{
					boolean aff[]= new boolean[4];
					for (int i1=0; i1<4; i1++)
						aff[i1]= false;
					for (int i1=1; i1<=4; i1++)
						switch(i1)
						{
							default:
							break;
							case 1:
				   				if (psAff[4]==1 && psAff[76]==1)
									aff[0]= true;
							break;
							case 2:
				   				if (psAff[22]==1)
									aff[1]= true;
							break;
							case 3:
				   				if (psAff[58]==1)
									aff[2]= true;
							break;					
							case 4:
				   				if (psAff[40]==1)
									aff[3]= true;
							break;					                                                        
						}
                                        int affIndex= -1;
					Random random = new Random();
					affIndex= random.nextInt(4);
					while (aff[affIndex]==false)
						affIndex= random.nextInt(4);
					if (affIndex==0 && (aff[1] || aff[2] || aff[3]))
					{
						aff[0]= false;
						while (aff[affIndex]==false)
							affIndex= random.nextInt(4);
					}
                                       switch(affIndex)
					{
						default:
						break;
						case 0:
							for(int k4 = 0; k4 < 80; k4++)
							{
                    			F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 4) * (k4 - 4)) / (2D * (d * d)));
                    			F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 76) * (k4 - 76)) / (2D * (d * d)));
                    		}
						break;
						case 1:
						    for(int k3 = 0; k3 < 80; k3++)
                    			F6.act[k3][0] += 0.04 * Math.exp((double)(-1 * (k3 - 22) * (k3 - 22)) / (2D * (d * d)));
		                break;
						case 2:
			                for(int k4 = 0; k4 < 80; k4++)
			                    F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 58) * (k4 - 58)) / (2D * (d * d)));
		                break;
						case 3:
			                for(int k4 = 0; k4 < 80; k4++)
			                    F6.act[k4][0] += 0.04 * Math.exp((double)(-1 * (k4 - 40) * (k4 - 40)) / (2D * (d * d)));
		                break;                                
					}		

}
}
        for(int i7 = 0; i7 < 80; i7++)
        {
            F1.act[i7][0] = psAff[i7];
            F3.act[i7][0] = 0.0D;
            F4.act[i7][0] = 0.0D;
            F7.act[i7][0] = 0.0D; 
            F14.act[i7][0] = 0.0D; 
            F19.act[i7][0] = 0.0D;
            F20.act[i7][0] = 0.0D;
        }
    }

    public void compute_curiosity(int i, boolean aflag[])
    {
        double d1 = 3D;
        double ad[] = new double[80];
        for(int l = 0; l < 80; l++)
            F14.act[l][0] = 0.0D;
        double d= 0D;
        if (Rat.testing)
            d= CURIOSITY_TESTING;
        else if (Rat.habituation)
            d= CURIOSITY_HABITUATION;
        else
            d= CURIOSITY;
        d1= CORRIDOR_SLOPE;
        for(int i1 = 0; i1 < 80; i1++)
            ad[i1] = Math.exp((double)(-1 * (i1 - 40) * (i1 - 40)) / (2D * (d1 * d1)));
        d1 = 3D;
        for(int k3 = 0; k3 < 8; k3++)
        {
            int k = i - k3;
            int j = (k + 4) % 8;
            if(j < 0)
                j += 8;
            switch(j)
            {
            default:
                break;

            case 1:
                if(F1.act[13][0] < ad[13] || aflag[k3])
                    break;
                for(int l1 = 0; l1 < 80; l1++)
                    F14.act[l1][0] += d * Math.exp((double)(-1 * (l1 - 13) * (l1 - 13)) / (2D * (d1 * d1)));

                break;

            case 2:
                if(F1.act[22][0] < ad[22] || aflag[k3])
                    break;
                for(int i2 = 0; i2 < 80; i2++)
                    F14.act[i2][0] += d * Math.exp((double)(-1 * (i2 - 22) * (i2 - 22)) / (2D * (d1 * d1)));

                break;

            case 3:
                if(F1.act[31][0] < ad[31] || aflag[k3])
                    break;
                for(int j2 = 0; j2 < 80; j2++)
                    F14.act[j2][0] += d * Math.exp((double)(-1 * (j2 - 31) * (j2 - 31)) / (2D * (d1 * d1)));

                break;

            case 4:
                if(F1.act[40][0] < ad[40] || aflag[k3])
                    break;
                for(int k2 = 0; k2 < 80; k2++)
                    F14.act[k2][0] += d * Math.exp((double)(-1 * (k2 - 40) * (k2 - 40)) / (2D * (d1 * d1)));

                break;

            case 5:
                if(F1.act[49][0] < ad[49] || aflag[k3])
                    break;
                for(int l2 = 0; l2 < 80; l2++)
                    F14.act[l2][0] += d * Math.exp((double)(-1 * (l2 - 49) * (l2 - 49)) / (2D * (d1 * d1)));

                break;

            case 6:
                if(F1.act[58][0] < ad[58] || aflag[k3])
                    break;
                for(int i3 = 0; i3 < 80; i3++)
                    F14.act[i3][0] += d * Math.exp((double)(-1 * (i3 - 58) * (i3 - 58)) / (2D * (d1 * d1)));

                break;

            case 7: 
                if(F1.act[67][0] < ad[67] || aflag[k3])
                    break;
                for(int j3 = 0; j3 < 80; j3++)
                    F14.act[j3][0] += d * Math.exp((double)(-1 * (j3 - 67) * (j3 - 67)) / (2D * (d1 * d1)));

                break;
            }
        }
    }

    public void WG_influence(int i, double ad[], boolean flagIni[], double psAff[])
    {
        double d = 0.0D;
        boolean flag = false;
        double d1 = 0.0D;
        double d3 = 3D;
        double ad1[] = new double[8];
        double ad2[] = new double[8];
        double d9 = 0.0D;
        double ad3[] = new double[8];
        for(int j = 0; j < 80; j++)
        {
            F19.act[j][0] = 0.0D;
            wg_mot[j] = 0.0D;
        }
        for(int l3 = 0; l3 < 8; l3++)
            d9 += ad[l3];

        if(d9 > 0.0D)
        {
            for(int i4 = 0; i4 < 8; i4++)
                if(ad[i4] != 0.0D)
                {
                    int j4 = i - i4;
                    int k4 = (j4 + 4) % 8;
                    if(k4 < 0)
                        k4 += 8;
                    switch(k4)
                    {
                    default:
                        break;

                    case 0:
                        ad3[0] = ad[i4];
                        if(ad[i4] != 0.0D)
                        {
                            for(int k = 0; k < 80; k++)
                            {
                                F19.act[k][0] += ad[i4] * Math.exp((double)(-1 * (k - 4) * (k - 4)) / (2D * (d3 * d3)));
                                F19.act[k][0] += ad[i4] * Math.exp((double)(-1 * (k - 76) * (k - 76)) / (2D * (d3 * d3)));
                            }

                        }
                        break;

                    case 1: 
                        ad3[1] = ad[i4];
                        if(ad[i4] == 0.0D)
                            break;
                        for(int l = 0; l < 80; l++)
                            F19.act[l][0] += ad[i4] * Math.exp((double)(-1 * (l - 13) * (l - 13)) / (2D * (d3 * d3)));

                        break;

                    case 2: 
                        ad3[2] = ad[i4];
                        if(ad[i4] == 0.0D)
                            break;
                        for(int i1 = 0; i1 < 80; i1++)
                            F19.act[i1][0] += ad[i4] * Math.exp((double)(-1 * (i1 - 22) * (i1 - 22)) / (2D * (d3 * d3)));

                        break;

                    case 3: 
                        ad3[3] = ad[i4];
                        if(ad[i4] == 0.0D)
                            break;
                        for(int j1 = 0; j1 < 80; j1++)
                            F19.act[j1][0] += ad[i4] * Math.exp((double)(-1 * (j1 - 31) * (j1 - 31)) / (2D * (d3 * d3)));

                        break;

                    case 4: 
                        ad3[4] = ad[i4];
                        if(ad[i4] == 0.0D)
                            break;
                        for(int k1 = 0; k1 < 80; k1++)
                            F19.act[k1][0] += ad[i4] * Math.exp((double)(-1 * (k1 - 40) * (k1 - 40)) / (2D * (d3 * d3)));

                        break;

                    case 5: 
                        ad3[5] = ad[i4];
                        if(ad[i4] == 0.0D)
                            break;
                        for(int l1 = 0; l1 < 80; l1++)
                            F19.act[l1][0] += ad[i4] * Math.exp((double)(-1 * (l1 - 49) * (l1 - 49)) / (2D * (d3 * d3)));

                        break;

                    case 6: 
                        ad3[6] = ad[i4];
                        if(ad[i4] == 0.0D)
                            break;
                        for(int i2 = 0; i2 < 80; i2++)
                            F19.act[i2][0] += ad[i4] * Math.exp((double)(-1 * (i2 - 58) * (i2 - 58)) / (2D * (d3 * d3)));

                        break;

                    case 7: 
                        ad3[7] = ad[i4];
                        if(ad[i4] == 0.0D)
                            break;
                        for(int j2 = 0; j2 < 80; j2++)
                            F19.act[j2][0] += ad[i4] * Math.exp((double)(-1 * (j2 - 67) * (j2 - 67)) / (2D * (d3 * d3)));

                        break;
                    }
                }
            double d5 = 0.0D;
            double d6 = 0.0D;
            for(int k2 = 0; k2 < 8; k2++)
            {
                ad1[k2] = 0.0D;
                ad2[k2] = 0.0D;
            }

            double d7 = Math.toRadians(45D);
            ad1[0] = ad3[0];
            ad2[0] = 0.0D;
            ad1[1] = ad3[1] * Math.cos(d7);
            ad2[1] = ad3[1] * Math.sin(d7);
            ad1[2] = 0.0D;
            ad2[2] = ad3[2];
            ad1[3] = -1D * ad3[3] * Math.cos(d7);
            ad2[3] = ad3[3] * Math.sin(d7);
            ad1[4] = -1D * ad3[4];
            ad2[4] = 0.0D;
            ad1[5] = -1D * ad3[5] * Math.cos(d7);
            ad2[5] = -1D * ad3[5] * Math.sin(d7);
            ad1[6] = 0.0D;
            ad2[6] = -1D * ad3[6];
            ad1[7] = ad3[7] * Math.cos(d7);
            ad2[7] = -1D * ad3[7] * Math.sin(d7);
            if(ad2[2] > 0.0D && ad2[6] < 0.0D && ad1[0] == 0.0D && ad1[1] == 0.0D && ad2[1] == 0.0D && ad1[3] == 0.0D && ad2[3] == 0.0D && ad1[4] == 0.0D && ad1[5] == 0.0D && ad2[5] == 0.0D && ad1[7] == 0.0D && ad2[7] == 0.0D)
            {
                compute_centroid(ad3, false);
            } else
            {
                for(int l2 = 0; l2 < 80; l2++)
                    F3.act[l2][0] = 0.0D;

                for(int i3 = 0; i3 < 8; i3++)
                {
                    d6 += ad1[i3];
                    d5 += ad2[i3];
                }

                double d8 = Math.toDegrees(Math.atan2(d6, d5));
                double d2 = Math.sqrt(d6 * d6 + d5 * d5);
                d8 *= -1D;
                if(d8 < 0.0D)
                    d8 += 360D;
                if(d8 == 360D)
                    d8 = 0.0D;
                d8 /= 5D;
                int l4 = ((int)d8 + 18) % 72;
                if(l4 < 0)
                    l4 += 72;
                l4 += 4;
                double d4 = 3D;
                if(d2 > 0.0D)
                {
                    for(int j3 = 0; j3 < 80; j3++)
                    {
                        F3.act[j3][0] = d2 * Math.exp((double)(-1 * (j3 - l4) * (j3 - l4)) / (2D * (d4 * d4)));
                        if(l4 == 4)
                            F3.act[j3][0] += d2 * Math.exp((double)(-1 * (j3 - 76) * (j3 - 76)) / (2D * (d4 * d4)));
                    }

                }
            }
        }
        double mayor = 0.0D;
        int pos = -1;
        for(int x = 0; x < 80; x++)
        {
            if(F3.act[x][0] > mayor)
            {
                mayor = F3.act[x][0];
                pos = x;
            }
        }
        if (pos != -1 && psAff[pos] <= 0.60D)
        {
            int dirGauss= -1;
            for (int a= 0; a<8; a++)
            {
                if(flagIni[a]== true)
                {
                    dirGauss= a;
                }
            }
            int rot = (i - dirGauss + 4) % 8;
            if(rot < 0)
                 rot += 8;
            double expec= 0.0D;
            for (int b=0; b<8; b++)
                expec+= ad[b];
            for(int j = 0; j < 80; j++)
                 F3.act[j][0] = 0.0D;
            switch(rot)
            {
                    default:
                    break;
                    case 0:
                            for(int k = 0; k < 80; k++)
                            {
                                F3.act[k][0] += expec * Math.exp((double)(-1 * (k - 4) * (k - 4)) / (2D * (d3 * d3)));
                                F3.act[k][0] += expec * Math.exp((double)(-1 * (k - 76) * (k - 76)) / (2D * (d3 * d3)));
                            }
                        break;
                    case 1: 
                        for(int l = 0; l < 80; l++)
                            F3.act[l][0] += expec * Math.exp((double)(-1 * (l - 13) * (l - 13)) / (2D * (d3 * d3)));
                        break;
                    case 2: 
                        for(int i1 = 0; i1 < 80; i1++)
                            F3.act[i1][0] += expec * Math.exp((double)(-1 * (i1 - 22) * (i1 - 22)) / (2D * (d3 * d3)));
                        break;
                    case 3: 
                        for(int j1 = 0; j1 < 80; j1++)
                            F3.act[j1][0] += expec * Math.exp((double)(-1 * (j1 - 31) * (j1 - 31)) / (2D * (d3 * d3)));
                        break;
                    case 4: 
                        for(int k1 = 0; k1 < 80; k1++)
                            F3.act[k1][0] += expec * Math.exp((double)(-1 * (k1 - 40) * (k1 - 40)) / (2D * (d3 * d3)));
                        break;
                    case 5: 
                        for(int l1 = 0; l1 < 80; l1++)
                            F3.act[l1][0] += expec * Math.exp((double)(-1 * (l1 - 49) * (l1 - 49)) / (2D * (d3 * d3)));
                        break;
                    case 6: 
                        for(int i2 = 0; i2 < 80; i2++)
                            F3.act[i2][0] += expec * Math.exp((double)(-1 * (i2 - 58) * (i2 - 58)) / (2D * (d3 * d3)));
                        break;
                    case 7: 
                        for(int j2 = 0; j2 < 80; j2++)
                            F3.act[j2][0] += expec * Math.exp((double)(-1 * (j2 - 67) * (j2 - 67)) / (2D * (d3 * d3)));
                        break;
               }
        }
        d = WG_NOISE;
        for(int k3 = 0; k3 < 80; k3++)
            wg_mot[k3] = F3.act[k3][0] + Math.random() * d;
    }

    public void compute_centroid(double ad[], boolean flag)
    {
        boolean flag1 = false;
        int k2 = 0;
        double d = 0.0D;
        double d1 = 0.0D;
        double d2 = 0.0D;
        double d3 = 3D;
        double d4 = 0.0D;
        double d5 = 0.0D;
        double ad1[] = new double[80];
        if(flag)
        {
            d3 = 8D;
            for(int i = 0; i < 80; i++)
            {
                F4.act[i][0] = 0.0D;
                ad1[i] = 0.0D;
            }

        } else
        {
            d3 = 3D;
            for(int j = 0; j < 80; j++)
            {
                F3.act[j][0] = 0.0D;
                ad1[j] = 0.0D;
            }

        }
        for(int k = 0; k < 8; k++)
            switch(k)
            {
            case 0: 
                ad1[4] = ad[k] / 2D;
                ad1[76] = ad[k] / 2D;
                break;

            case 1: 
                ad1[13] = ad[k];
                break;

            case 2: 
                ad1[22] = ad[k];
                break;

            case 3: 
                ad1[31] = ad[k];
                break;

            case 4: 
                ad1[40] = ad[k];
                break;

            case 5: 
                ad1[49] = ad[k];
                break;

            case 6: 
                ad1[58] = ad[k];
                break;

            case 7: 
                ad1[67] = ad[k];
                break;
            }

        for(int l = 0; l < 80; l++)
        {
            if(ad1[l] < 0.0D)
                ad1[l] = 0.0D;
            if(ad1[l] != 0.0D)
            {
                d2 += ad1[l];
                k2++;
            }
        }

        if(k2 > 0)
            d = d2;
        else
            d = 0.0D;
        d2 = 0.0D;
        for(int i1 = 0; i1 < 80; i1++)
            if(ad1[i1] > d4)
                d4 = ad1[i1];

        d5 *= -1D;
        for(int j1 = 0; j1 < 80; j1++)
            if(d4 > 0.0D && ad1[j1] > 0.0D)
                ad1[j1] /= d4;

        for(int k1 = 0; k1 < 80; k1++)
            if(ad1[k1] > 0.0D)
            {
                d1 += ad1[k1] * (double)(k1 - 40);
                d2 += ad1[k1];
            }

        if(d2 != 0.0D)
        {
            int j2 = (int)(d1 / d2);
            j2 += 40;
            if(d > 0.0D)
                if(flag)
                {
                    for(int l1 = 0; l1 < 80; l1++)
                        F4.act[l1][0] = d * Math.exp((double)(-1 * (l1 - j2) * (l1 - j2)) / (2D * (d3 * d3)));

                } else
                {
                    for(int i2 = 0; i2 < 80; i2++)
                        F3.act[i2][0] = d * Math.exp((double)(-1 * (i2 - j2) * (i2 - j2)) / (2D * (d3 * d3)));

                }
        }
    }
     
	public void computeInputToActionSelection()
    {
        Angle = 0;
        
        F1.max = 0.0D;
        F1.min = 0.0D;
        F3.max = 0.0D;
        F3.min = 0.0D;
        F4.max = 0.0D;
        F4.min = 0.0D;
        F5.max = 0.0D;
        F5.min = 0.0D;
        F6.max = 0.0D;
        F6.min = 0.0D;
        F7.max = 0.0D;
        F7.min = 0.0D;
        F14.max = 0.0D;
        F14.min = 0.0D;
        F18.max = 0.0D;
        F18.min = 0.0D;
        F19.max = 0.0D;
        F19.min = 0.0D;
        F20.max = 0.0D;
        F20.min = 0.0D;
        for(int k = 0; k < 80; k++)
            affAux[k]= F1.act[k][0];
        for(int k = 0; k <= 8; k++)
            affAux[k]-= (affAux[k]/2);
        for(int k = 72; k <= 79; k++)
            affAux[k]-= (affAux[k]/2);
        if (Rat.habituation)
        {
        for(int k = 0; k < 80; k++)
            u[k] = F1.act[k][0] + F6.act[k][0] + F14.act[k][0];
        }
        else if (Rat.training)
        {
        for(int k = 0; k < 80; k++)
            u[k] = F1.act[k][0] + F6.act[k][0] + F14.act[k][0] + wg_mot[k];
        }
        else if (Rat.testing)
        {
        for(int k = 0; k < 80; k++)
            u[k] = affAux[k] + F6.act[k][0] + F14.act[k][0] + wg_mot[k];
        }
        double d4 = 0.0D;
        int i4 = -1;
        for(int l1 = 0; l1 < 80; l1++)
        {
            U[l1] = 0.0D;
            if(u[l1] > d4)
            {
                d4 = u[l1];
                i4 = l1;
            }
        }
        if(d4 > 0.0D)
        {
            for(int i2 = 0; i2 < 80; i2++)
	            if(i2 != i4)
                    U[i2] = 0.0D;
                else
                    U[i2] = 1.0D;
        }
        for(int j2 = 0; j2 < 80; j2++)
        {
            F5.act[j2][0] = Gradient[j2] * U[j2];
            Angle += (int)F5.act[j2][0];
        }
        for(int k2 = 0; k2 < 80; k2++)
            F5.act[k2][0] = u[k2];
        for(int l2 = 0; l2 < 80; l2++)
        {
            if(F1.max < F1.act[l2][0])
                F1.max = F1.act[l2][0];
            if(F3.max < F3.act[l2][0])
                F3.max = F3.act[l2][0];
            if(F6.max < F6.act[l2][0])
                F6.max = F6.act[l2][0];
            if(F18.max < F18.act[l2][0])
                F18.max = F18.act[l2][0];
            if(F14.max < F14.act[l2][0])
                F14.max = F14.act[l2][0];
            if(F19.max < F19.act[l2][0])
                F19.max = F19.act[l2][0];
            if(F20.max < F20.act[l2][0])
                F20.max = F20.act[l2][0];
            if(F7.min > F7.act[l2][0])
                F7.min = F7.act[l2][0];
            if(F4.min > F4.act[l2][0])
                F4.min = F4.act[l2][0];
            if(F19.min > F19.act[l2][0])
                F19.min = F19.act[l2][0];
            if(F3.min > F3.act[l2][0])
                F3.min = F3.act[l2][0];
            double d = F5.act[l2][0];
            if(d < 0.0D)
                d *= -1D;
            if(F5.max < d)
                F5.max = d;
        }

        for(int i3 = 0; i3 < 80; i3++)
            if(F5.max > 0.0D && F5.act[i3][0] < 0.0D)
                F5.act[i3][0] *= -1D;

        for(int j3 = 0; j3 < 80; j3++)
        {
            F19.act[j3][0] += -1D * F19.min;
            if(F19.max < F19.act[j3][0])
                F19.max = F19.act[j3][0];
            F4.act[j3][0] += -1D * F4.min;
            if(F4.max < F4.act[j3][0])
                F4.max = F4.act[j3][0];
            F7.act[j3][0] += -1D * F7.min;
            if(F7.max < F7.act[j3][0])
                F7.max = F7.act[j3][0];
        }
    }
     
    public void simRun()
	{
		if (!inicio)
			turnToFoodAux= turnToFood.get();
		else
			inicio= false;
		
		currentDir= angle2direction(currentHeadAngleRat.get());	
		initConditions(turnToFoodAux,affPS.get());
		

		flagCyclingAuxI= flagCycling.get();
        for(int j1 = 0; j1 < 8; j1++)
        {
        	if (flagCyclingAuxI[j1] == 1)
            	flagCyclingAuxB[j1] = true;
            else
            	flagCyclingAuxB[j1] = false;
        }

		compute_curiosity(currentDir, flagCyclingAuxB);
                if (!Rat.habituation)
                {
                    WG_influence(currentDir, expCycling.get(),flagCyclingAuxB, affPS.get());
                }
		
		computeInputToActionSelection();

		int aux=0;
		while (aux<80 && U[aux]!=1)
			aux++;
		if (aux<80)
			System.out.println("Mayor-U["+aux+"]= "+U[aux]);
                
		AngleToGo.set(Angle);
                
                int m,n;
                for (m=0, n=4; m<5 && n<80; m++,n+=18)
                {
                    curiosityAux[m]= F14.act[n][0];
                    foodAux[m]= F6.act[n][0];
                    wgAux[m]= F3.act[n][0];
                    sumAux[m]= u[n];
                }
                wg.set(wgAux);
                curiosity.set(curiosityAux);
                food.set(foodAux);
                sum.set(sumAux);
	} //simRun
}	
