/*
 * Este mdulo sirve de interfaz entre el ambiente virtual y el modelo.
 * Autor: Alejandra Barrera
 * Fecha: 13 de mayo de 2005
 */

import nslj.src.system.*;
import nslj.src.cmd.*;
import nslj.src.lang.*;
import nslj.src.math.*;  
import nslj.src.display.*; 
import javax.vecmath.*;

public class World extends NslModule
{
    public NslDinDouble0 changeXRat;
    public NslDinDouble0 changeYRat;
    public NslDinDouble0 changeZRat;
    public NslDinInt0 newHeadAngleRat;
    public NslDinInt0 ActionTaken;
    public NslDoutInt0 currentHeadAngle;
    public NslDoutInt2 colorMatrixO;
    public NslDoutInt2 colorMatrixR;
    public NslDoutInt2 colorMatrixL;
    private WorldFrame world;
    private double x;
    private double y;
    private double z;
    private int headAngle;
    private int action;
    private int actionDegrees;
    private int colorMatrixOAux[][];
    private int colorMatrixRAux[][];
    private int colorMatrixLAux[][];
    private boolean inicio;
    private int dim1;
    private int dim2;
    private boolean foodRight;
    public static boolean initAnchorTesting;
    public static int testingTrialNumber;
    public static int trainingTrialNumber;

    public World(String nslName, NslModule nslParent, int d1, int d2, int angleInit)
    {
        super(nslName, nslParent);
        x= 0.0;
        y= 0.0;
        z= 0.0;
        initAnchorTesting= false;
        trainingTrialNumber=0;
        testingTrialNumber=0;
        action= 4;
        headAngle= angleInit;
        dim1= d1;
        dim2= d2;
        inicio= true;
        foodRight= false;
        changeXRat=new NslDinDouble0("changeXRat",this); 
        changeYRat=new NslDinDouble0("changeYRat",this); 
        changeZRat=new NslDinDouble0("changeZRat",this); 
        newHeadAngleRat=new NslDinInt0("newHeadAngleRat",this); 
        ActionTaken= new NslDinInt0("ActionTaken", this);
        colorMatrixO= new NslDoutInt2("colorMatrixO",this,dim1,dim2);
        colorMatrixR= new NslDoutInt2("colorMatrixR",this,dim1,dim2);
        colorMatrixL= new NslDoutInt2("colorMatrixL",this,dim1,dim2);
        currentHeadAngle= new NslDoutInt0("currentHeadAngle",this);
        
        colorMatrixOAux= new int[dim1][dim2];
        colorMatrixRAux= new int[dim1][dim2];
        colorMatrixLAux= new int[dim1][dim2];
        world = new WorldFrame();
        world.show();
    }
    
    public void simRun()
    {
        if (!inicio)
        {
            x= changeXRat.get();
            y= changeYRat.get();
            z= changeZRat.get();
            headAngle= newHeadAngleRat.get();
            action= ActionTaken.get();
        }
        else
            inicio= false;

       switch(action)
       {
           case 0: actionDegrees= -180; break;
           case 1: actionDegrees= -135; break;
           case 2: actionDegrees= -90; break;
           case 3: actionDegrees= -45; break;
           case 4: actionDegrees= 0; break;
           case 5: actionDegrees= 45; break;
           case 6: actionDegrees= 90; break;
           case 7: actionDegrees= 135; break;
           default: actionDegrees= 0; break;
       }
            world.rotateRobot((float)actionDegrees);  

       if (Rat.training && trainingTrialNumber==1)
       {
            System.out.println("******** ENTRENAMIENTO ********");
            world.startRobot(0f,-0.025f,0.495f);
            try {
                Thread.sleep(750);
            }    
            catch(Exception e) {
                System.out.println(e);
            }
       }
       else if (Rat.testing && Rat.newTrial && testingTrialNumber==1)
       {
            world.moveRobot(0f,0f,1f,0f);
            try {
                Thread.sleep(750);
            }    
            catch(Exception e) {
                System.out.println(e);
            }
       }
       else if (Rat.testing && Rat.newTrial && testingTrialNumber==2)
       {
            world.moveRobot(-0.6f,0f,0.5f,0f);
            try {
                Thread.sleep(750);
            }    
            catch(Exception e) {
                System.out.println(e);
            }
       }
       else if (Rat.testing && Rat.newTrial && testingTrialNumber==3)
       {
            world.moveRobot(-1.1f,0f,0.5f,0f);
            try {
                Thread.sleep(750);
            }    
            catch(Exception e) {
                System.out.println(e);
            }
       }
       else if (Rat.testing && Rat.newTrial && testingTrialNumber==4)
       {
            world.moveRobot(-1.1f,0f,0f,0f);
            try {
                Thread.sleep(750);
            }    
            catch(Exception e) {
                System.out.println(e);
            }
       }            
       else if (Rat.testing && Rat.newTrial && testingTrialNumber==5)
       {
           System.out.println("********TERMINA LA FASE DE PRUEBA********");
       }                        
       else            
            world.moveRobot(x,y,z,(float)actionDegrees);

       for (int i=0; i<dim1; i++)
            for (int j=0; j<dim1; j++)
            {
                colorMatrixOAux[i][j]= 0;
                colorMatrixRAux[i][j]= 0;
                colorMatrixLAux[i][j]= 0;
            }
       
       
      try {
            Thread.sleep(1000);
       }
       catch(Exception e) {
            System.out.println(e);
       }
        colorMatrixOAux= world.getColorMatrix();
        world.rotateRobotCamera(90f);
        
        
               try {
            Thread.sleep(750);
       }
       catch(Exception e) {
            System.out.println(e);
       }
        
        
        colorMatrixRAux= world.getColorMatrix();
        world.rotateRobotCamera(180f);
               try {
            Thread.sleep(750);
       }
       catch(Exception e) {
            System.out.println(e);
       }

        colorMatrixLAux= world.getColorMatrix();
        world.rotateRobotCamera(90f); 
              try {
            Thread.sleep(750);
       }
       catch(Exception e) {
            System.out.println(e);
       }
      
        colorMatrixO.set(colorMatrixOAux);
        colorMatrixR.set(colorMatrixRAux);
        colorMatrixL.set(colorMatrixLAux);
        currentHeadAngle.set(headAngle);
    }
}
